@@ -13,17 +13,20 @@ field, check OAuthBackend class for details on how to extend it.
 """
 import cgi
 from urllib import urlencode
-from urllib2 import urlopen
+from urllib2 import urlopen, HTTPError
 
 from django.utils import simplejson
 from django.contrib.auth import authenticate
 
 from social_auth.backends import BaseOAuth2, OAuthBackend, USERNAME
 from social_auth.utils import sanitize_log_data, setting, log
+from social_auth.backends.exceptions import AuthException, AuthCanceled, \
+                                            AuthFailed, AuthTokenError
 
 
 # Facebook configuration
 FACEBOOK_ME = 'https://graph.facebook.com/me?'
+ACCESS_TOKEN = 'https://graph.facebook.com/oauth/access_token?'
 
 
 class FacebookBackend(OAuthBackend):
@@ -56,10 +59,12 @@ class FacebookAuth(BaseOAuth2):
     def get_scope(self):
         return setting('FACEBOOK_EXTENDED_PERMISSIONS', [])
 
-    def user_data(self, access_token):
+    def user_data(self, access_token, *args, **kwargs):
         """Loads user data from service"""
         data = None
-        url = FACEBOOK_ME + urlencode({'access_token': access_token})
+        params = setting('FACEBOOK_PROFILE_EXTRA_PARAMS', {})
+        params['access_token'] = access_token
+        url = FACEBOOK_ME + urlencode(params)
 
         try:
             data = simplejson.load(urlopen(url))
@@ -67,37 +72,49 @@ class FacebookAuth(BaseOAuth2):
             extra = {'access_token': sanitize_log_data(access_token)}
             log('error', 'Could not load user data from Facebook.',
                 exc_info=True, extra=extra)
+        except HTTPError:
+            extra = {'access_token': sanitize_log_data(access_token)}
+            log('error', 'Error validating access token.',
+                exc_info=True, extra=extra)
+            raise AuthTokenError(self)
         else:
             log('debug', 'Found user data for token %s',
-                sanitize_log_data(access_token),
-                extra=dict(data=data))
+                sanitize_log_data(access_token), extra={'data': data})
         return data
 
     def auth_complete(self, *args, **kwargs):
         """Completes loging process, must return user instance"""
-        if 'code' in self.data:
-            url = 'https://graph.facebook.com/oauth/access_token?' + \
-                  urlencode({'client_id': setting('FACEBOOK_APP_ID'),
-                             'redirect_uri': self.redirect_uri,
-                             'client_secret': setting('FACEBOOK_API_SECRET'),
-                             'code': self.data['code']})
+        if 'code' not in self.data:
+            if self.data.get('error') == 'access_denied':
+                raise AuthCanceled(self)
+            else:
+                raise AuthException(self)
+
+        url = ACCESS_TOKEN + urlencode({
+            'client_id': setting('FACEBOOK_APP_ID'),
+            'redirect_uri': self.redirect_uri,
+            'client_secret': setting('FACEBOOK_API_SECRET'),
+            'code': self.data['code']
+        })
+        try:
             response = cgi.parse_qs(urlopen(url).read())
-            access_token = response['access_token'][0]
-            data = self.user_data(access_token)
-            if data is not None:
-                if 'error' in data:
-                    error = self.data.get('error') or 'unknown error'
-                    raise ValueError('Authentication error: %s' % error)
-                data['access_token'] = access_token
-                # expires will not be part of response if offline access
-                # premission was requested
-                if 'expires' in response:
-                    data['expires'] = response['expires'][0]
-            kwargs.update({'response': data, self.AUTH_BACKEND.name: True})
-            return authenticate(*args, **kwargs)
-        else:
-            error = self.data.get('error') or 'unknown error'
-            raise ValueError('Authentication error: %s' % error)
+        except HTTPError:
+            raise AuthFailed(self, 'There was an error authenticating the app')
+
+        access_token = response['access_token'][0]
+        data = self.user_data(access_token)
+
+        if data is not None:
+            data['access_token'] = access_token
+            # expires will not be part of response if offline access
+            # premission was requested
+            if 'expires' in response:
+                data['expires'] = response['expires'][0]
+
+        kwargs.update({'auth': self,
+                       'response': data,
+                       self.AUTH_BACKEND.name: True})
+        return authenticate(*args, **kwargs)
 
     @classmethod
     def enabled(cls):
